
#define MODULE nvs

#include <caf/events/module_state_event.h>
#include <zephyr/logging/log.h>

#include <zephyr/sys/reboot.h>
#include <zephyr/device.h>
#include <string.h>

#include <zephyr/drivers/flash.h>
#include <zephyr/storage/flash_map.h>
#include <zephyr/fs/nvs.h>

#include <cjson.h>

LOG_MODULE_REGISTER(MODULE, LOG_LEVEL_DBG);

static struct nvs_fs fs;

#define NVS_PARTITION			custom_nvs_partition
#define NVS_PARTITION_DEVICE	FIXED_PARTITION_DEVICE(NVS_PARTITION)
#define NVS_PARTITION_OFFSET	FIXED_PARTITION_OFFSET(NVS_PARTITION)

#define CONFIG_DATA_ID 1

#define DEFAULT_CONFIG_STR "{\"a\":2, \"b\":\"string\", \"c\":[1,2,3]}"
static char default_conf[4096];
static char my_buf[4096];

static cJSON* config;

static struct flash_pages_info info;

int setup_flash() 
{
	int rc = 0;
	fs.flash_device = NVS_PARTITION_DEVICE;
	if (!device_is_ready(fs.flash_device)) {
		LOG_INF("Flash device %s is not ready", fs.flash_device->name);
		return -1;
	}
	fs.offset = NVS_PARTITION_OFFSET;
	rc = flash_get_page_info_by_offs(fs.flash_device, fs.offset, &info);
	if (rc) {
		LOG_ERR("Unable to get page info");
		return rc;
	}
	fs.sector_size = info.size;
	fs.sector_count = 4U;

	rc = nvs_mount(&fs);
	if (rc) {
		LOG_INF("Flash Init failed");
		return rc;
	}
	return rc;
}

static void read_config() {
	strcpy(default_conf, DEFAULT_CONFIG_STR); // automatically null-terminated
	size_t load_amt = strlen(default_conf);

	// Read Out Current Contents
	int rc = nvs_read(&fs, CONFIG_DATA_ID, &my_buf, load_amt+1);
	if (rc > 0) {
		/* item was found, show it */
		LOG_INF("loaded %d chars from ID[%d]", load_amt, CONFIG_DATA_ID);
		LOG_INF("Data Loaded: \n%s", my_buf);
        config = cJSON_Parse(my_buf);
		LOG_INF("Data Parsed: \n%s", cJSON_Print(config));
	}
	else {
		LOG_WRN("Couldn't Read from NVS: %d", rc);
	}

	// Write new content
	LOG_INF("Writing default_conf, len = %d", strlen(default_conf));
	rc = nvs_write(&fs, CONFIG_DATA_ID, &default_conf, load_amt+1 );
	if(rc >= 0) {
		LOG_INF("Wrote %d chars to nvs", rc);
	}
	else {
		LOG_ERR("Failed to write to NVS (%d)", rc);
	}
}

static bool app_event_handler(const struct app_event_header *aeh)
{
    if(is_module_state_event(aeh)){
        struct module_state_event *ev = cast_module_state_event(aeh);
        if(check_state(ev, MODULE_ID(main), MODULE_STATE_READY)) {
            int rc = setup_flash();
			if(rc) {
				LOG_ERR("Setup Flash Failed (%d)", rc);
			}
			k_msleep(100);
            read_config();
            return false;
        }
    }
    return false;
}

APP_EVENT_LISTENER(MODULE, app_event_handler);
APP_EVENT_SUBSCRIBE(MODULE, module_state_event);
